/*
 *
 *  * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 *  * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 *
 *
 */

package ru.ulstu.tsMethods.exponential;

import ru.ulstu.models.TimeSeries;
import ru.ulstu.models.exceptions.ModelingException;
import ru.ulstu.tsMethods.TimeSeriesMethod;
import ru.ulstu.tsMethods.exponential.param.Alpha;
import ru.ulstu.tsMethods.exponential.param.Beta;
import ru.ulstu.tsMethods.exponential.param.ExponentialMethodParamValue;

import java.util.ArrayList;
import java.util.List;

public class AddTrendNoSeason extends TimeSeriesMethod {
    private final ExponentialMethodParamValue<Alpha> alpha;
    private final ExponentialMethodParamValue<Beta> beta;
    private final List<Double> sComponent = new ArrayList<>();
    private final List<Double> tComponent = new ArrayList<>();

    public AddTrendNoSeason(TimeSeries timeSeries,
                            ExponentialMethodParamValue<Alpha> alpha,
                            ExponentialMethodParamValue<Beta> beta) throws ModelingException {
        super(timeSeries);
        this.alpha = alpha;
        this.beta = beta;
    }

    @Override
    protected TimeSeries getModelOfValidTimeSeries() throws ModelingException {
        sComponent.clear();
        tComponent.clear();
        sComponent.add(originalTimeSeries.getFirstValue().getValue());
        tComponent.add(originalTimeSeries.getValues().get(1).getValue() - originalTimeSeries.getValues().get(0).getValue());
        TimeSeries model = new TimeSeries("Model of " + originalTimeSeries.getName());
        model.addValue(originalTimeSeries.getFirstValue());
        //выполняется проход модели по сглаживанию
        for (int t = 1; t < originalTimeSeries.getValues().size(); t++) {
            sComponent.add(alpha.getDoubleValue() * originalTimeSeries.getNumericValue(t)
                    + (1 - alpha.getDoubleValue())
                    * (sComponent.get(t - 1) - tComponent.get(t - 1)));

            tComponent.add(beta.getDoubleValue()
                    * (sComponent.get(t) - sComponent.get(t - 1))
                    + (1 - beta.getDoubleValue()) * tComponent.get(t - 1));
            model.addValue(originalTimeSeries.getValues().get(t), sComponent.get(sComponent.size() - 1));
        }
        return model;
    }

    @Override
    protected TimeSeries makeForecast(TimeSeries forecast) {
        for (int t = 1; t < forecast.getLength(); t++) {
            forecast.getValues().get(t).setValue(sComponent.get(sComponent.size() - 1) + tComponent.get(tComponent.size() - 1) * t);
        }
        return forecast;
    }
}
