/*
 *
 *  * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 *  * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 *
 *
 */

package ru.ulstu.tsMethods.exponential;

import ru.ulstu.models.TimeSeries;
import ru.ulstu.models.exceptions.ModelingException;
import ru.ulstu.tsMethods.TimeSeriesMethod;
import ru.ulstu.tsMethods.exponential.param.Alpha;
import ru.ulstu.tsMethods.exponential.param.ExponentialMethodParamValue;

import java.util.ArrayList;
import java.util.List;

public class NoTrendNoSeason extends TimeSeriesMethod {
    private final ExponentialMethodParamValue<Alpha> alpha;
    private final List<Double> sComponent = new ArrayList<>();

    public NoTrendNoSeason(TimeSeries timeSeries, ExponentialMethodParamValue<Alpha> alpha) throws ModelingException {
        super(timeSeries);
        this.alpha = alpha;
    }

    @Override
    protected TimeSeries getModelOfValidTimeSeries() throws ModelingException {
        sComponent.clear();
        sComponent.add(originalTimeSeries.getFirstValue().getValue());
        TimeSeries model = new TimeSeries("Model of " + originalTimeSeries.getName());
        model.addValue(originalTimeSeries.getFirstValue());
        //выполняется проход модели по сглаживанию
        for (int t = 1; t < originalTimeSeries.getValues().size(); t++) {
            sComponent.add(sComponent.get(t - 1)
                    + alpha.getDoubleValue()
                    * (originalTimeSeries.getNumericValue(t) - sComponent.get(t - 1)));
            model.addValue(originalTimeSeries.getValues().get(t), sComponent.get(sComponent.size() - 1));
        }
        return model;
    }

    @Override
    protected TimeSeries makeForecast(TimeSeries forecast) {
        for (int t = 1; t < forecast.getLength(); t++) {
            forecast.getValues().get(t).setValue(forecast.getValues().get(t - 1).getValue());
        }
        return forecast;
    }
}
